{
  Simple/Single Form
  2003-04 -

  2004-01-25 AlreadyRun: MutexȂƂ0ł͂Ȃ-1Ԃ悤ɂ
}

unit SForm;

interface

uses Windows, Messages;

type
  TNotifyEvent = procedure (Sender: TObject) of object;

  TWinControl = class(TObject)
    Handle: HWND;
    function GetClientRect: TRect;
    procedure SetClientRect(Value: TRect);
    function GetText: string;
    procedure SetText(const Value: string);
    procedure SetBounds(ALeft, ATop, AWidth, AHeight: Integer); virtual;
    procedure SetSize(AWidth, AHeight: Integer); virtual;
    procedure SetPosition(ALeft, ATop: Integer); virtual;
    procedure Show; virtual;
    procedure Hide; virtual;
    procedure Update; virtual;
    procedure Invalidate; virtual;
    property ClientRect: TRect read GetClientRect write SetClientRect;
    property Text: string read GetText write SetText;
  end;

  TCommonControl = class(TWinControl)
  protected
    OrgWndProc: TFNWndProc;
    procedure CreateWnd(Parent: HWND; ALeft, ATop, AWidth, AHeight: Integer); virtual;
    procedure WndProc(var Message: TMessage); virtual;
  end;

  TSForm = class(TWinControl)
  protected
    SaveWP: TWindowPlacement;
    SaveWS: Longint;
    SaveStayOnTop: Boolean;
    FRegisteredClass: Boolean;
    FName: string;
    FWndClassName: string;
    FStayOnTop: Boolean;
    FFullScreen: Boolean;
    FBorder: Boolean;
    procedure SetStayOnTop(Value: Boolean);
    procedure SetFullScreen(Value: Boolean);
    procedure SetBorder(Value: Boolean);
  public
    Style: DWORD;
    ExStyle: DWORD;
    Brush: HBRUSH;
    Icon: HICON;
    Cursor: HCURSOR;
    ClassStyle: UINT;
    property Name: string read FName;
    property WndClassName: string read FWndClassName;
    property StayOnTop: Boolean read FStayOnTop write SetStayOnTop;
    property FullScreen: Boolean read FFullScreen write SetFullScreen;
    property Border: Boolean read FBorder write SetBorder;
    constructor Create(const AName: string); virtual;
    procedure CreateWnd(ALeft, ATop, AWidth, AHeight: Integer);
    procedure Show; override;
    procedure Close;
    procedure WndProc(var Message: TMessage); virtual;
  end;

function MessageLoop: Integer;
function MessageLoopWithAccel(hWnd: HWND; hAccel: HACCEL): Integer;
function MessageLoopWithAccelAndDialog(hWnd: HWND; hAccel: HACCEL; var hDlg: HWND): Integer;

function AlreadyRun(const AppName: string): HWND;
function GetSysErrMsg(Code: Integer): string;
procedure FormatMsgBox(hWnd: HWND; const Caption: string; const Text: string;
  const Args: array of string; Flags: Integer);
procedure SetWindowCenter(hWnd: HWND);
function OsVersionInfo: TOsVersionInfo;
procedure AboutBox(hWnd: HWND; const Caption: string; const Text: string);
procedure ShowMsg(const Msg: string);

var
  MsgMouseWheel: Integer; // for Win95

implementation

const
  PropName = 'OBJECT';

{ TWinControl }

function TWinControl.GetClientRect: TRect;
begin
  Windows.GetClientRect(Handle, Result);
end;

procedure TWinControl.SetClientRect(Value: TRect);
begin
  AdjustWindowRectEx(Value, GetWindowLong(Handle, GWL_STYLE),
    (GetParent(Handle) = 0) and (GetMenu(Handle) <> 0),
    GetWindowLong(Handle, GWL_EXSTYLE));
  with Value do
    SetSize(Right-Left, Bottom-Top);
end;

procedure TWinControl.Setbounds(ALeft, ATop, AWidth, AHeight: Integer);
begin
  MoveWindow(Handle, ALeft, ATop, AWidth, AHeight, True);
end;

procedure TWinControl.SetSize(AWidth, AHeight: Integer);
begin
  SetWindowPos(Handle, 0, 0, 0, AWidth, AHeight, SWP_NOZORDER or SWP_NOMOVE or SWP_NOACTIVATE);
end;

procedure TWinControl.SetPosition(ALeft, ATop: Integer);
begin
  SetWindowPos(Handle, 0, ALeft, ATop, 0, 0, SWP_NOZORDER or SWP_NOSIZE or SWP_NOACTIVATE);
end;

function TWinControl.GetText: string;
begin
  SetLength(Result, GetWindowTextLength(Handle));
  GetWindowText(Handle, PChar(Result), Length(Result)+1); // +1null terminate
end;

procedure TWinControl.SetText(const Value: string);
begin
  SetWindowText(Handle, PChar(Value));
end;

procedure TWinControl.Show;
begin
  ShowWindow(Handle, SW_SHOW);
end;

procedure TWinControl.Hide;
begin
  ShowWindow(Handle, SW_HIDE);
end;

procedure TWinControl.Update;
begin
  UpdateWindow(Handle);
end;

procedure TWinControl.Invalidate;
begin
  InvalidateRect(Handle, nil, True);
end;

{ TCommonControl }

function ComCtlWindowProc(hWnd: HWND; Msg: UINT; wParam: WPARAM; lParam: LPARAM): LRESULT; stdcall;
var
  CommonControl: TCommonControl;
  Message: TMessage;
begin
  CommonControl := TCommonControl(GetProp(hWnd, PropName));
  if Assigned(CommonControl) then
  begin
    Message.Msg := Msg;
    Message.WParam := wParam;
    Message.LParam := lParam;
    Message.Result := 0;
    CommonControl.WndProc(Message);
    Result := Message.Result;
  end else
    Result := 0;
end;

procedure TCommonControl.WndProc(var Message: TMessage);
begin
  with Message do
  begin
    case Msg of
      WM_NCDESTROY:
        begin
          RemoveProp(Handle, PropName);
          SetWindowLong(Handle, GWL_WNDPROC, Integer(@OrgWndProc));
          Result := 0;
        end;
    else
      Result := CallWindowProc(OrgWndProc, Handle, Msg, wParam, lParam);
    end;
  end;
end;

procedure TCommonControl.CreateWnd(Parent: HWND; ALeft, ATop, AWidth, AHeight: Integer);
begin
  SetProp(Handle, PropName, Integer(Self));
  OrgWndProc := TFnWndProc(SetWindowLong(Handle, GWL_WNDPROC, Integer(@ComCtlWindowProc)));
end;

{ TSForm }

function SFormWindowProc(hWnd: HWND; Msg: UINT; wParam: WPARAM; lParam: LPARAM): LRESULT; stdcall;
var
  Form: TSForm;
  Message: TMessage;
begin
  case Msg of
    WM_NCCREATE:
      begin
        SetProp(hWnd, PropName, Integer(PCreateStruct(lParam).lpCreateParams));
        TSForm(PCreateStruct(lParam).lpCreateParams).Handle := hWnd;
        Result := DefWindowProc(hWnd, Msg, wParam, lParam);
      end;
    WM_NCDESTROY:
      begin
        RemoveProp(hWnd, PropName);
        Result := DefWindowProc(hWnd, Msg, wParam, lParam);
      end;
  else
    Form := TSForm(GetProp(hWnd, PropName));
    if Assigned(Form) then
    begin
      Message.Msg := Msg;
      Message.WParam := wParam;
      Message.LParam := lParam;
      Message.Result := 0;
      Form.WndProc(Message);
      Result := Message.Result;
    end else
      Result := 0;
  end;
end;

procedure TSForm.WndProc(var Message: TMessage);
begin
  with Message do
    Result := DefWindowProc(Handle, Msg, WParam, LParam);
end;

constructor TSForm.Create(const AName: string);
begin
  FName := AName;
  FWndClassName := 'T' + FName + 'Class';
  FRegisteredClass := False;
  FBorder := True;
  Brush := COLOR_3DFACE+1;
  Icon := LoadIcon(HInstance, 'MAINICON');
  Cursor := LoadCursor(0, IDC_ARROW);
  Style := WS_OVERLAPPEDWINDOW;
  ClassStyle := CS_DBLCLKS;
end;

procedure TSForm.SetStayOnTop(Value: Boolean);
begin
  FStayOnTop := Value;
  if Handle = 0 then
    Exit;
  if FStayOnTop then
    SetWindowPos(Handle, HWND_TOPMOST, 0, 0, 0, 0, SWP_NOMOVE or SWP_NOSIZE)
  else
    SetWindowPos(Handle, HWND_NOTOPMOST, 0, 0, 0, 0, SWP_NOMOVE or SWP_NOSIZE);
end;

procedure TSForm.SetFullScreen(Value: Boolean);
begin
  FFullScreen := Value;
  if Handle = 0 then
    Exit;
  if FFullScreen then
  begin
    SaveStayOnTop := StayOnTop;
    StayOnTop := True;
    FillChar(SaveWP, SizeOf(SaveWP), 0);
    SaveWP.length := SizeOf(SaveWP);
    GetWindowPlacement(Handle, @SaveWP);
    SaveWS := GetWindowLong(Handle, GWL_STYLE);
    SetWindowLong(Handle, GWL_STYLE, WS_OVERLAPPEDWINDOW
      and not WS_CAPTION and not WS_THICKFRAME and not WS_SYSMENU);
    ShowWindow(Handle, SW_SHOWNA);
    MoveWindow(Handle, 0, 0, GetSystemMetrics(SM_CXSCREEN),
      GetSystemMetrics(SM_CYSCREEN), True);
  end else
  begin
    SetWindowLong(Handle, GWL_STYLE, SaveWS and not WS_MAXIMIZE);
    ShowWindow(Handle, SW_SHOWNA);
    SetWindowPlacement(Handle, @SaveWP);
    StayOnTop := SaveStayOnTop;
  end;
end;

// StyleWS_OVERLAPPEDWINDOWȊÔƂ͐삵Ȃ
procedure TSForm.SetBorder(Value: Boolean);
var
  rc: TRect;
begin
  FBorder := Value;
  rc := ClientRect;
  if FBorder then
    SetWindowLong(Handle, GWL_STYLE, WS_OVERLAPPEDWINDOW)
  else
    SetWindowLong(Handle, GWL_STYLE, WS_OVERLAPPEDWINDOW
      and not WS_CAPTION and not WS_THICKFRAME and not WS_SYSMENU);
  ShowWindow(Handle, SW_SHOWNA);
  ClientRect := rc;
end;

procedure TSForm.CreateWnd(ALeft, ATop, AWidth, AHeight: Integer);
var
  wc: TWndClass;
begin
  if not FRegisteredClass then
  begin
    FillChar(wc, SizeOf(wc), 0);
    wc.style         := ClassStyle;
    wc.lpfnWndProc   := @SFormWindowProc;
    wc.hInstance     := HInstance;
    wc.hIcon         := Icon;
    wc.hCursor       := Cursor;
    wc.hbrBackGround := Brush;
    wc.lpszClassName := PChar(FWndClassName);
    if RegisterClass(wc) = 0 then
      Exit;
    FRegisteredClass := True;
  end;

  Handle := 0;
  Handle := CreateWindowEx(ExStyle, PChar(FWndClassName), PChar(Name), Style,
    ALeft, ATop, AWidth, AHeight, 0, 0, HInstance, Self);
end;

procedure TSForm.Show;
begin
  ShowWindow(Handle, SW_SHOW);
  UpdateWindow(Handle);
end;

procedure TSForm.Close;
begin
  SendMessage(Handle, WM_SYSCOMMAND, SC_CLOSE, 0);
end;

{ Function and Procedure }

function MessageLoop: Integer;
var
  Msg: TMsg;
  Ret: Integer;
begin
  while True do
  begin
    Ret := Integer(GetMessage(Msg, 0, 0, 0));
    if (Ret = 0) or (Ret = -1) then
      Break
    else begin
      TranslateMessage(Msg);
      DispatchMessage(Msg);
    end;
  end;
  Result := Msg.wParam;
end;

function MessageLoopWithAccel(hWnd: HWND; hAccel: HACCEL): Integer;
var
  Msg: TMsg;
  Ret: Integer;
begin
  while True do
  begin
    Ret := Integer(GetMessage(Msg, 0, 0, 0));
    if (Ret = 0) or (Ret = -1) then
      Break
    else if not LongBool(TranslateAccelerator(hWnd, hAccel, Msg)) then
    begin
      TranslateMessage(Msg);
      DispatchMessage(Msg);
    end;
  end;
  Result := Msg.wParam;
end;

function MessageLoopWithAccelAndDialog(hWnd: HWND; hAccel: HACCEL; var hDlg: HWND): Integer;
var
  Msg: TMsg;
  Ret: Integer;
begin
  while True do
  begin
    Ret := Integer(GetMessage(Msg, 0, 0, 0));
    if (Ret = 0) or (Ret = -1) then
      Break
    else if (hDlg = 0) or not IsDialogMessage(hDlg, Msg) then
    begin
      if not LongBool(TranslateAccelerator(hWnd, hAccel, Msg)) then
      begin
        TranslateMessage(Msg);
        DispatchMessage(Msg);
      end;
    end;
  end;
  Result := Msg.wParam;
end;

// ɋNĂăEBhEƂ:       EBhEnh
// ɋNĂăEBhEȂƂ: 0
// NĂȂƂ:                           -1
function AlreadyRun(const AppName: string): HWND;
var
  Mutex: THandle;
  ClassName: string;
begin
  // Mutexnh̓vZXIOSIclose
  Mutex := CreateMutex(nil, True, PChar(AppName+'Mutex'));
  if Mutex = 0 then
  begin
    MessageBox(GetDesktopWindow, 'CreateMutex failed.', PChar(AppName), MB_OK or MB_ICONSTOP);
    Halt(255);
  end;
  if GetLastError = ERROR_ALREADY_EXISTS then
  begin
    ClassName := 'T' + AppName + 'Class';
    Result := FindWindow(PChar(ClassName), nil);
  end else
    Result := -1;
end;

function GetSysErrMsg(Code: Integer): string;
var
  pBuf: PChar;
begin
  FormatMessage(FORMAT_MESSAGE_FROM_SYSTEM or FORMAT_MESSAGE_ALLOCATE_BUFFER, nil,
    Code, (SUBLANG_DEFAULT shl 10) or LANG_NEUTRAL, @pBuf, 0, nil);
  Result := pBuf;
  LocalFree(HLOCAL(pBuf));
end;

procedure FormatMsgBox(hWnd: HWND; const Caption: string; const Text: string;
  const Args: array of string; Flags: Integer);
type
  TPointerArray = array[0..0] of Pointer;
  PPointerArray = ^TPointerArray;
var
  pBuf: PChar;
  i: Integer;
  TmpArg: PPointerArray;
begin
  if not IsWindowVisible(hWnd) then
    hWnd := GetDesktopWindow;

  GetMem(TmpArg, SizeOf(Pointer) * (High(Args)+1));
  try
    for i := 0 to High(Args) do
      TmpArg[i] := PChar(Args[i]);
    FormatMessage(FORMAT_MESSAGE_FROM_STRING or FORMAT_MESSAGE_ARGUMENT_ARRAY or
      FORMAT_MESSAGE_ALLOCATE_BUFFER, PChar(Text), 0, 0, @pBuf, 0, TmpArg);
  finally
    FreeMem(TmpArg);
  end;
  MessageBox(hWnd, pBuf, PChar(Caption), Flags);
  LocalFree(HLOCAL(pBuf));
end;

procedure SetWindowCenter(hWnd: HWND);
var
  rc, drc: TRect;
begin
  GetWindowRect(GetDesktopWindow, drc);
  GetWindowRect(hWnd, rc);
  SetWindowPos(hWnd, 0, (drc.Right - (rc.Right - rc.Left)) div 2,
    (drc.Bottom - (rc.Bottom - rc.Top)) div 2, 0, 0,
    SWP_NOZORDER or SWP_NOSIZE or SWP_NOACTIVATE);
end;

function OsVersionInfo: TOsVersionInfo;
begin
  FillChar(Result, SizeOf(Result), 0);
  Result.dwOSVersionInfoSize := SizeOf(TOsVersionInfo);
  GetVersionEx(Result);
end;

procedure AboutBox(hWnd: HWND; const Caption: string; const Text: string);
const
  ABIconW: PWideChar = 'MAINICON';
  ABIconA: PChar = 'MAINICON';
var
  mbp: TMsgBoxParams;
begin
  FillChar(mbp, SizeOf(mbp), 0);
  with mbp do
  begin
    cbSize := SizeOf(mbp);
    hwndOwner := hWnd;
    hInstance := MainInstance;
    lpszText := PChar(Text);
    lpszCaption := PChar(Caption);
    dwStyle := MB_USERICON;
    // NTłWideCharœnȂƂ(ANSI version APIȂ̂Ɂc)
    if OsVersionInfo.dwPlatformId = VER_PLATFORM_WIN32_NT then
       lpszIcon := Pointer(ABIconW)
    else
       lpszIcon := ABIconA;
  end;
  MessageBoxIndirect(mbp);
end;

procedure ShowMsg(const Msg: string);
begin
  MessageBox(GetDesktopWindow, PChar(Msg), PChar(ParamStr(0)), MB_OK);
end;

begin
  MsgMouseWheel := RegisterWindowMessage('MSWHEEL_ROLLMSG'); // for Win95
end.
