(*
    NkVp - Video Player
    Copyright (C) 2003 Nobuyuki Honda (Ferdia) <ferdia@imasy.or.jp>

    Compiler: Delphi 3.01a
*)

program NkVp;

uses
  Windows, Messages, SysUtils, ShellAPI, ActiveX, CommDlg, DShow,
  VideoPlayer, SUtil, SForm, SCmd, SConfFile, SList;

{$R *.res}

const
  AppName = 'NkVp';
  Version = '0.5 (2004-02-01)';
  Copyright = 'Copyright (C) 2003-2004 Nobuyuki Honda (Ferdia)'#13#10#13#10+
    'This software is provided under the zlib/libpng license.'#13#10;

type
  TFileList = class(TSStrList)
    CurPos: Integer;
    procedure Clear;
    function CurFileName: string;
  end;

  TAspcetMode = (amAuto, amOrg, am43, am169);

  TMainForm = class(TSForm)
    ModeStr: string;
    TimeStr: string;
    FormatStr: string;
    AspectMode: TAspcetMode;
    Zoom: Integer;
    DispWidth: Integer;
    DispHeight: Integer;
    Mute: Boolean;
    SaveMode: TVPModes;
    RepeatFlg: Boolean;
    HideCursorCount: Integer;
    OldPt: TPoint;
    FHideCursor: Boolean;
    constructor Create(const AName: string); override;
    procedure SetHideCursor(Value: Boolean);
    procedure CheckHideCursor;
    procedure DoCmd(const CmdName: string);
    procedure SetTitle;
    procedure SetFormatStr;
    procedure FitWindow;
    procedure ZoomVideo(Value: Integer);
    procedure ShowAbout;
    procedure PopupMenu(const pt: TPoint);
    procedure PopupFileListMenu(const pt: TPoint);
    procedure DrawStatusBar(dc: HDC; rc: TRect);
    procedure OnTimer;
    procedure OnEndPlay;
    procedure OnDropFiles(hDrop: THandle);
    procedure OnCommand(Id: Integer);
    procedure WndProc(var Message: TMessage); override;
    procedure SaveScreenCapture;
    procedure LoadFile(const FileName: string);
    function GetVideoDispWidth: Integer;
    procedure UpdateVideoWindowSize;
    procedure OnCopyData(pCopyData: PCopyDataStruct);
    property HideCursor: Boolean read FHideCursor write SetHideCursor;
  end;

  TWaveFmtTable = record
    Tag: Integer;
    Name: string;
  end;

var
  CmdList: array[0..24] of TCmd = (
    (Name: 'Pause'),
    (Name: 'Stop'),
    (Name: 'SkipPrev'),
    (Name: 'SkipNext'),
    (Name: 'Mute'),
    (Name: 'FullScreen'),
    (Name: 'StayOnTop'),
    (Name: 'Zoom50'),
    (Name: 'Zoom100'),
    (Name: 'Zoom150'),
    (Name: 'Zoom200'),
    (Name: 'Zoom300'),
    (Name: 'Zoom400'),
    (Name: 'SeekForward'),
    (Name: 'SeekBack'),
    (Name: 'SeekForwardFast'),
    (Name: 'SeekBackFast'),
    (Name: 'Aspect43'),
    (Name: 'Aspect169'),
    (Name: 'AspectOrg'),
    (Name: 'AspectAuto'),
    (Name: 'Repeat'),
    (Name: 'ScreenCapture'),
    (Name: 'About'),
    (Name: 'Exit')
  );

  MenuTable: array[0..29] of TMenuItem = (
    (Caption: '&Play / Pause';              CmdName: 'Pause'),
    (Caption: '&Stop';                      CmdName: 'Stop'),
    (Caption: 'Skip &Previous';             CmdName: 'SkipPrev'),
    (Caption: 'Skip &Next';                 CmdName: 'SkipNext'),
    (Caption: '&Mute';                      CmdName: 'Mute'),
    (Caption: '&Repeat';                    CmdName: 'Repeat'),
    (Caption: '-'),
    (Caption: 'Zoom &50%';                  CmdName: 'Zoom50'),
    (Caption: 'Zoom &100%';                 CmdName: 'Zoom100'),
    (Caption: 'Zoom 150%(&Q)';              CmdName: 'Zoom150'),
    (Caption: 'Zoom &200%';                 CmdName: 'Zoom200'),
    (Caption: 'Zoom &300%';                 CmdName: 'Zoom300'),
    (Caption: 'Zoom &400%';                 CmdName: 'Zoom400'),
    (Caption: '-'),
    (Caption: 'Aspect A&uto';               CmdName: 'AspectAuto'),
    (Caption: 'Aspect Ori&ginal';           CmdName: 'AspectOrg'),
    (Caption: 'Aspect 4:3(&E)';             CmdName: 'Aspect43'),
    (Caption: 'Aspect 16:9(&W)';            CmdName: 'Aspect169'),
    (Caption: '-'),
    (Caption: 'Full &Screen';               CmdName: 'FullScreen'),
    (Caption: 'Stay &On Top';               CmdName: 'StayOnTop'),
    (Caption: '-'),
    (Caption: 'Save Screen &Capture As...'; CmdName: 'ScreenCapture'),
    (Caption: '-'),
    (Caption: '&Filter Properties';         CmdName: '{FilterMenu'),
    (Caption: '';                           CmdName: '}'),
    (Caption: '-'),
    (Caption: '&About';                     CmdName: 'About'),
    (Caption: '-'),
    (Caption: 'E&xit';                      CmdName: 'Exit')
  );

const
  DefaultKeys: array[0..28*2-1] of string = (
    'Enter',   'FullScreen',
    'Esc',     'Stop',
    'Space',   'Pause',
    'Left',    'SeekBack',
    'S-Left',  'SeekBackFast',
    'Right',   'SeekForward',
    'S-Right', 'SeekForwardFast',
    'Num1',    'Zoom100',
    'Num2',    'Zoom200',
    'Num3',    'Zoom300',
    'Num4',    'Zoom400',
    'Num5',    'Zoom50',
    '1',       'Zoom100',
    '2',       'Zoom200',
    '3',       'Zoom300',
    '4',       'Zoom400',
    '5',       'Zoom50',
    'A',       'AspectAuto',
    'B',       'SkipPrev',
    'E',       'Aspect43',
    'M',       'Mute',
    'N',       'SkipNext',
    'O',       'AspectOrg',
    'P',       'SkipPrev',
    'Q',       'Zoom150',
    'R',       'Repeat',
    'T',       'StayOnTop',
    'W',       'Aspect169'
  );

  WaveFormatTable: array[0..2] of TWaveFmtTable = (
    (Tag: 1;  Name: 'PCM'),
    (Tag: 2;  Name: 'MSADPCM'),
    (Tag: 85; Name: 'MP3')
  );

  IDM_FILTERPROP_FIRST = 4000;
  IDM_FILTERPROP_LAST = IDM_FILTERPROP_FIRST + 255;
  IDM_FILELIST_FIRST = 5000;
  IDM_FILELIST_LAST = IDM_FILELIST_FIRST + 10000;

  WM_USER_LOADFILE = WM_USER + 1;

  KeyMapFile = 'keymap';
  CaptureBmpFile = 'nkvpcapt.bmp';

  MsgSaveScreenCaptureError = 'Cannot save screen capture.%n%n%1%n%n';
  MsgCannotPlay = 'Cannot play.%n%n%1%n%n';

  StatBarHeight = 13;
  WheelSeek = 1000; // zC[gpseek(msec)
  FastSeek = 10000; // V[Nseek(msec)
  ZoomTable: array [0..8] of Integer = (25, 50, 66, 100, 125, 150, 200, 300, 400);

  HideCursorDelay = 3; // sec

var
  MainForm: TMainForm;
  Video: TVideoPlayer;
  Cmd: TSCmd;
  ProfDir: string;
  FileList: TFileList;

{ TFileList }

procedure TFileList.Clear;
begin
  CurPos := 0;
  inherited;
end;

function TFileList.CurFileName: string;
begin
  Result := Strings[CurPos];
end;

{ TMainForm }

function GetTimeStr(n: Integer): string;
var
  t: Integer;
  v: array[0..2] of Integer;
begin
  SetLength(Result, 8);
  t := n div 1000;
  v[0] := t div 3600;
  v[1] := t mod 3600 div 60;
  v[2] := t mod 60;
  wvsprintf(PChar(Result), '%02d:%02d:%02d', @v);
end;

constructor TMainForm.Create(const AName: string);
begin
  inherited;
  ClassStyle := CS_HREDRAW or CS_VREDRAW or CS_DBLCLKS;
  Style := WS_OVERLAPPEDWINDOW or WS_CLIPCHILDREN;
  ExStyle := WS_EX_ACCEPTFILES;
  Brush := GetStockObject(BLACK_BRUSH);
  Zoom := 100;
end;

procedure TMainForm.ShowAbout;
var
  s: string;
begin
  Str(Video.Aspect:4:4, s);
  AboutBox(Handle, 'About '+AppName,
    AppName+' '+Version+#13#10#13#10+Copyright+#13#10+
   'Profile Folder: '+ProfDir+#13#10#13#10+
   'Window Size: '+SStr(DispWidth)+'x'+SStr(DispHeight)+#13#10+
   'File Aspcet: '+s);
end;

function TMainForm.GetVideoDispWidth: Integer;
begin
  case AspectMode of
    amOrg:  Result := Video.Width;
    amAuto: Result := Round(Video.Width / Video.Aspect);
    am43:   Result := Round(Video.Height * 4/3);
    am169:  Result := Round(Video.Height * 16/9);
  else
    Result := Video.Width;
  end;
  if Result = 0 then
    Result := 320;
end;

procedure TMainForm.FitWindow;
var
  rc, wrc: TRect;
  w, h: Integer;
begin
  if IsZoomed(Handle) then
    ShowWindow(Handle, SW_RESTORE);
  w := GetVideoDispWidth;
  h := Video.Height;
  rc.Left := 0;
  rc.Top := 0;
  rc.Right := (w * Zoom) div 100;
  rc.Bottom := (h * Zoom) div 100;
  wrc := rc;
  AdjustWindowRect(wrc, GetWindowLong(Handle, GWL_STYLE), False);
  with wrc do
    SetSize(Right-Left, Bottom-Top+StatBarHeight);
  if Video.Loaded then
    Video.SetWindowPosition(rc.left, rc.top, rc.right, rc.bottom);
end;

procedure TMainForm.SetFormatStr;
var
  s: string;
  i: Integer;
begin
  FormatStr := '';
  if Video.Loaded then
  begin
    for i := 0 to High(WaveFormatTable) do
    begin
      if WaveFormatTable[i].Tag = Video.WaveFormat.wFormatTag then
      begin
        FormatStr := WaveFormatTable[i].Name + ' ';
        Break;
      end;
    end;
    if Video.WaveFormat.nChannels = 2 then
      s := 'st '
    else
      s := '';
    FormatStr := FormatStr +
      SStr(Video.WaveFormat.nSamplesPerSec div 1000)+'kHz '+
      SStr(Video.WaveFormat.nAvgBytesPerSec*8 div 1000) + 'k ' + s +
      Copy(PChar(@Video.BitmapInfoHeader.biCompression), 1, 4) + ' ';
  end;
end;

procedure TMainForm.UpdateVideoWindowSize;
var
  rc: TRect;
  w, h: Integer;
begin
  if Video.Loaded then
  begin
    rc := ClientRect;
    Dec(rc.bottom, StatBarHeight);
    w := GetVideoDispWidth;
    h := Video.Height;
    if h = 0 then
      Exit;
    if w*rc.Bottom > h*rc.Right then
    begin
      DispWidth := rc.Right;
      DispHeight := (h * DispWidth) div w;
    end
    else begin
      DispHeight := rc.Bottom;
      DispWidth := (w * DispHeight) div h;
    end;
    if DispHeight > DispWidth then
      Zoom := (DispWidth * 100) div w
    else
      Zoom := (DispHeight * 100) div h;
    Video.SetWindowPosition((rc.Right-DispWidth) div 2,
      (rc.Bottom-DispHeight) div 2, DispWidth, DispHeight);
    SetTitle;
  end;
end;

procedure TMainForm.SetTitle;
var
  s: string;
begin
  if Video.Loaded then
  begin
    with Video do
    begin
      Str(Fps :0:3, s);
      Text := BaseName(FileName)+' ('+SStr(Width)+'x'+SStr(Height)+
        ' '+s+'fps) '+SStr(Zoom)+'% - '+AppName;
    end;
  end else
    Text := AppName;
end;

procedure TMainForm.SaveScreenCapture;
var
  ofn: TOpenFilename;
  buf: array[0..MAX_PATH] of Char;
  r: HResult;
begin
  Video.PauseOnly;
  OnTimer;
  Update; // ĕ`悵ĂȂƍcۂ
  FillChar(ofn, SizeOf(ofn), 0);
  with ofn do
  begin
    lStructSize := SizeOf(ofn);
    hWndOwner := Handle;
    lpstrFilter := 'All Files'#0'*.*'#0#0;
    lpstrFile := buf;
    nMaxFile := SizeOf(buf);
    Flags := OFN_OVERWRITEPROMPT;
  end;
  buf := CaptureBmpFile;
  if GetSaveFileName(ofn) then
  begin
    r := Video.WriteBitmapBits(ofn.lpstrFile);
    //r := E_UNEXPECTED; // test
    if r <> S_OK then
      FormatMsgBox(Handle, AppName, MsgSaveScreenCaptureError+GetDShowErrMsg(r),
        [ofn.lpstrFile], MB_ICONEXCLAMATION);
  end;
end;

procedure TMainForm.OnEndPlay;
begin
  if RepeatFlg then
  begin
    Video.Position := 0;
    Video.Play;
  end
  else if FileList.CurPos < FileList.Count - 1 then
  begin
    Inc(FileList.CurPos);
    PostMessage(Handle, WM_USER_LOADFILE, 0, 0);
  end;
end;

procedure TMainForm.OnTimer;
var
  rc: TRect;
begin
  TimeStr := FormatStr + GetTimeStr(Video.Position) + ' / ' + GetTimeStr(Video.Length) + ' ';
  if FileList.Count > 1 then
    ModeStr := ' ' + SStr(FileList.CurPos+1) + '/' + SStr(FileList.Count) + ':'
  else
    ModeStr := '';
  case Video.Mode of
    vpPause: ModeStr := ModeStr + ' PAUSE ';
    vpStop:  ModeStr := ModeStr + ' STOP ';
  else
    ModeStr := ModeStr + ' ';
  end;
  if RepeatFlg then
    ModeStr := ModeStr + 'REPEAT ';
  if Mute then
    ModeStr := ModeStr + 'MUTE';
  rc := ClientRect;
  rc.Top := rc.Bottom - StatBarHeight;
  InvalidateRect(Handle, @rc, False);
end;

procedure TMainForm.OnDropFiles(hDrop: THandle);
var
  FileName: string;
  cnt, i: Integer;
begin
  FileList.Clear;
  cnt := DragQueryFile(hDrop, $FFFFFFFF, nil, 0);
  for i := 0 to cnt - 1 do
  begin
    SetLength(FileName, DragQueryFile(hDrop, i, nil, 0)+1);
    DragQueryFile(hDrop, i, PChar(FileName), Length(FileName));
    // null terminateȂ̂PCharɃLXg
    FileList.Add(PChar(FileName));
  end;
  FileList.Sort;

  DragFinish(hDrop);

  LoadFile(FileList.CurFileName);
  Zoom := 100;
  FitWindow;
end;

procedure TMainForm.DrawStatusBar(dc: HDC; rc: TRect);
var
  src, rc2: TRect;
  bbr, gbr: HBRUSH;
  font: HFONT;
  mdc: HDC;
  bmp, oldbmp: HBITMAP;
begin
  src.Left := 0;
  src.Top := 0;
  src.Right :=  rc.Right - rc.Left;
  src.Bottom := rc.Bottom - rc.Top;
  mdc := CreateCompatibleDC(dc);
  bmp := CreateCompatibleBitmap(dc, src.Right, src.Bottom);
  oldbmp := SelectObject(mdc, bmp);

  bbr := GetStockObject(BLACK_BRUSH);
  gbr := GetStockObject(GRAY_BRUSH);
  FrameRect(mdc, src, gbr);
  Inc(src.Top);
  FillRect(mdc, src, bbr);
  if Video.Loaded and (Video.Length > 0) then
    rc2.Left := Trunc(Video.Position / Video.Length * rc.Right) - 2
  else
    rc2.Left := -2;
  rc2.Right := rc2.Left + 4 + 1;
  rc2.Top := 2;
  rc2.Bottom := src.Bottom - 1;
  FillRect(mdc, rc2, gbr);
  Inc(rc2.Left ,2);
  rc2.Right := rc2.Left + 1;
  rc2.Bottom := rc2.Top + 2;
  FillRect(mdc, rc2, bbr);
  rc2.Bottom := src.Bottom - 1;
  rc2.Top := rc2.Bottom - 2;
  FillRect(mdc, rc2, bbr);
  SetTextColor(mdc, RGB(0, 255, 255));
  SetBkMode(mdc, TRANSPARENT);
  font := SelectObject(mdc, GetStockObject(DEFAULT_GUI_FONT));
  DrawText(mdc, PChar(ModeStr), Length(ModeStr), src, DT_SINGLELINE or DT_VCENTER);
  DrawText(mdc, PChar(TimeStr), Length(TimeStr), src, DT_RIGHT or DT_SINGLELINE or DT_VCENTER);
  SelectObject(mdc, font);
  BitBlt(dc, rc.Left, rc.Top, rc.Right, rc.Bottom, mdc, 0, 0, SRCCOPY);
  SelectObject(mdc, oldbmp);
  DeleteObject(bmp);
  DeleteDC(mdc);
end;

procedure TMainForm.ZoomVideo(Value: Integer);
begin
  if FullScreen then
    FullScreen := False;
  Zoom := Value;
  FitWindow;
  SetTitle;
end;

procedure TMainForm.SetHideCursor(Value: Boolean);
begin
  if Value <> FHideCursor then
  begin
    FHideCursor := Value;
    ShowCursor(not FHideCursor);
  end;
end;

procedure TMainForm.DoCmd(const CmdName: string);
begin
  if CmdName = '' then
    Exit
  else if CmdName = 'Pause' then
  begin
    if Video.Mode = vpStop then
      Video.Play
    else
      Video.Pause;
  end
  else if CmdName = 'Stop' then
    Video.Stop
  else if CmdName = 'SkipPrev' then
  begin
    if FileList.CurPos > 0 then
    begin
      Dec(FileList.CurPos);
      LoadFile(FileList.CurFileName);
    end;
  end
  else if CmdName = 'SkipNext' then
  begin
    if FileList.CurPos < FileList.Count - 1 then
    begin
      Inc(FileList.CurPos);
      LoadFile(FileList.CurFileName);
    end;
  end
  else if Copy(CmdName, 1, 4) = 'Zoom' then
    ZoomVideo(SInt(Copy(CmdName, 5, Length(CmdName)-5+1)))
  else if CmdName = 'StayOnTop' then
    StayOnTop := not StayOnTop
  else if CmdName = 'FullScreen' then
  begin
    FullScreen := not FullScreen;
    if FullScreen then
      SetTimer(Handle, 1, 500, nil)
    else begin
      HideCursor := False;
      KillTimer(Handle, 1);
    end;
  end
  else if CmdName = 'Mute' then
  begin
    Mute := not Mute;
    if Mute then
      Video.Volume := -10000
    else
      Video.Volume := 0;
  end
  else if CmdName = 'SeekForward' then
    Video.Position := Video.Position + WheelSeek
  else if CmdName = 'SeekBack' then
    Video.Position := Video.Position - WheelSeek
  else if CmdName = 'SeekForwardFast' then
    Video.Position := Video.Position + FastSeek
  else if CmdName = 'SeekBackFast' then
    Video.Position := Video.Position - FastSeek
  else if CmdName = 'AspectAuto' then
  begin
    AspectMode := amAuto;
    UpdateVideoWindowSize;
  end
  else if CmdName = 'AspectOrg' then
  begin
    AspectMode := amOrg;
    UpdateVideoWindowSize;
  end
  else if CmdName = 'Aspect43' then
  begin
    AspectMode := am43;
    UpdateVideoWindowSize;
  end
  else if CmdName = 'Aspect169' then
  begin
    AspectMode := am169;
    UpdateVideoWindowSize;
  end
  else if CmdName = 'ScreenCapture' then
    SaveScreenCapture
  else if CmdName = 'Repeat' then
    RepeatFlg := not RepeatFlg
  else if CmdName = 'About' then
    ShowAbout
  else if CmdNAme = 'Exit' then
    Close;
end;

procedure TMainForm.OnCommand(Id: Integer);
var
  r: HRESULT;
begin
  case Id of
    IDM_FILTERPROP_FIRST .. IDM_FILTERPROP_LAST:
      begin
        r := Video.ShowFilterPropertyByName(Video.FilterNames[Id-IDM_FILTERPROP_FIRST]);
        if r <> 0 then
          FormatMsgBox(Handle, AppName, '%1%n%n'+GetDShowErrMsg(r),
            [Video.FilterNames[Id-IDM_FILTERPROP_FIRST]], MB_ICONEXCLAMATION);
      end;
    IDM_FILELIST_FIRST .. IDM_FILELIST_LAST:
      begin
        FileList.CurPos := Id-IDM_FILELIST_FIRST;
        LoadFile(FileList.CurFileName);
      end;
  else
    DoCmd(Cmd.Name(Id));
  end;
  OnTimer;
end;

procedure TMainForm.PopupMenu(const pt: TPoint);
var
  Menu, SubMenu: HMENU;
  i: Integer;
begin
  ResetMenuState(MenuTable);
  if not Video.Loaded then
    Cmd.DisableMenu(MenuTable, [
      'Pause',
      'Stop',
      'FullScreen',
      'Zoom50',
      'Zoom100',
      'Zoom150',
      'Zoom200',
      'Zoom300',
      'Zoom400',
      'ScreenCapture',
      '{FilterMenu'
    ]);

  Cmd.CheckMenu(MenuTable, 'FullScreen', FullScreen);
  Cmd.CheckMenu(MenuTable, 'StayOnTop', StayOnTop);
  Cmd.CheckMenu(MenuTable, 'Mute', Mute);
  Cmd.CheckMenu(MenuTable, 'Repeat', RepeatFlg);
  Cmd.CheckMenu(MenuTable, 'AspectAuto', AspectMode = amAuto);
  Cmd.CheckMenu(MenuTable, 'AspectOrg', AspectMode = amOrg);
  Cmd.CheckMenu(MenuTable, 'Aspect43', AspectMode = am43);
  Cmd.CheckMenu(MenuTable, 'Aspect169', AspectMode = am169);

  Menu := CreatePopupMenu;
  Cmd.GenerateMenu(Menu, MenuTable);
  SubMenu := GetSubMenu(Menu, GetMenuItemCount(Menu) - 5);
  for i := 0 to Video.FilterCount - 1 do
    AppendMenu(SubMenu, MF_STRING, IDM_FILTERPROP_FIRST+i, PChar(Video.FilterNames[i]));

  TrackPopupMenu(Menu, TPM_LEFTALIGN or TPM_LEFTBUTTON or TPM_RIGHTBUTTON,
    pt.x, pt.y, 0, Handle, nil);

  DestroyMenu(Menu);
end;

procedure TMainForm.PopupFileListMenu(const pt: TPoint);
var
  Menu: HMENU;
  i: Integer;
  s: string;
begin
  Menu := CreatePopupMenu;
  for i := 0 to FileList.Count - 1 do
  begin
    if i < 9 then
      s := '0&'
    else
      s := '';
    s := s + SStr(i+1) + ' ' + BaseName(FileList[i]);
    AppendMenu(Menu, MF_STRING, IDM_FILELIST_FIRST+i, PChar(s));
  end;
  CheckMenuItem(Menu, FileList.CurPos, MF_BYPOSITION or MF_CHECKED);

  TrackPopupMenu(Menu, TPM_LEFTALIGN or TPM_LEFTBUTTON or TPM_RIGHTBUTTON,
    pt.x, pt.y, 0, Handle, nil);

  DestroyMenu(Menu);
end;

procedure TMainForm.OnCopyData(pCopyData: PCopyDataStruct);
var
  FileName: string;
begin
  with pCopyData^ do
  begin
    if dwData <> 0 then
      Exit;
    SetLength(FileName, cbData);
    Move(lpData^, FileName[1], cbData);
    if IsIconic(Handle) then
      OpenIcon(Handle);

    FileList.Clear;
    FileList.Add(FileName);
    PostMessage(Handle, WM_USER_LOADFILE, 1, 0); //͕ʃXbh?Ȃ̂
  end;
end;

procedure TMainForm.CheckHideCursor;
var
  pt: TPoint;
begin
  GetCursorPos(pt);
  if (OldPt.x = pt.x) and (OldPt.y = pt.y) then
  begin
    if HideCursorCount = HideCursorDelay*2 then
      HideCursor := True;
    Inc(HideCursorCount);
  end
  else begin
    HideCursor := False;
    HideCursorCount := 0;
  end;
  OldPt := pt;
end;


procedure TMainForm.WndProc(var Message: TMessage);
var
  rc: TRect;
  ps: TPaintStruct;
  dc: HDC;
  pt, cpt: TPoint;
begin
  with Message do
  begin
    Result := 0;
    if (Msg = WM_MOUSEWHEEL) or ((Msg = MsgMouseWheel) and (MsgMouseWheel <> 0)) then
    begin
      if wParam > 0 then
      begin
        if GetKeyState(VK_SHIFT) < 0 then
          DoCmd('SeekBackFast')
        else
          DoCmd('SeekBack');
      end
      else begin
        if GetKeyState(VK_SHIFT) < 0 then
          DoCmd('SeekForwardFast')
        else
          DoCmd('SeekForward');
      end;
      Exit;
    end;
    case Msg of
      WM_USER_LOADFILE:
        begin
          LoadFile(FileList.CurFileName);
          if wParam = 1 then
          begin
            Zoom := 100;
            FitWindow;
          end;
        end;
      WM_TIMER:
        CheckHideCursor;
      WM_GETMINMAXINFO:
        with PMinMaxInfo(lParam)^ do
        begin
          ptMaxTrackSize.x := 4096;
          ptMaxTrackSize.y := 4096;
        end;
      WM_DROPFILES:
        OnDropFiles(wParam);
      WM_COMMAND:
        OnCommand(LoWord(wParam));
      WM_PAINT:
        begin
          dc := BeginPaint(Handle, ps);
          rc := ClientRect;
          //if not Video.Loaded then
          //  FillRect(dc, rc, GetStockObject(BLACK_BRUSH));
          rc.Top := rc.Bottom - StatBarHeight;
          DrawStatusBar(dc, rc);
          EndPaint(Handle, ps);
        end;
      WM_CONTEXTMENU:
        if GetContextMenuPos(Handle, LParam, 0, 0, pt) then
        begin
          rc := ClientRect;
          cpt := pt;
          ScreenToClient(Handle, cpt);
          if (cpt.y > (rc.Bottom - StatBarHeight)) and (cpt.y < rc.Bottom) then
            PopupFileListMenu(pt)
          else
            PopupMenu(pt)
        end else
          inherited;
      WM_MBUTTONUP:
        DoCmd('Stop');
      WM_MOUSEMOVE:
        if GetCapture = Handle then
        begin
          GetCursorPos(pt);
          ScreenToClient(Handle, pt);
          rc := ClientRect;
          Video.Position := Trunc(pt.x / rc.Right * Video.Length);
          OnTimer;
        end;
      WM_LBUTTONDOWN,
      WM_LBUTTONDBLCLK:
        begin
          rc := ClientRect;
          if HIWORD(LParam) < (rc.Bottom - StatBarHeight) then
            DoCmd('Pause')
          else begin
            SetCapture(Handle);
            SaveMode := Video.Mode;
            Video.PauseOnly;
            Video.Position := Trunc(LOWORD(LParam) / rc.Right * Video.Length);
            OnTimer;
          end;
        end;
      WM_LBUTTONUP:
        if GetCapture = Handle then
        begin
          ReleaseCapture;
          if SaveMode = vpPlay then
            Video.Play;
        end;
      WM_MOVE:
        if Video.Loaded then
          InvalidateRect(GetWindow(Handle, GW_CHILD), nil, False); // VideoWindowXV
      WM_SIZE:
        UpdateVideoWindowSize;
      WM_COPYDATA:
        begin
          OnCopyData(PCopyDataStruct(lParam));
          Result := Longint(True);
        end;
      WM_CLOSE:
        begin
          Video.PauseOnly;
          Video.FreeInterface;
          DestroyWindow(Handle);
        end;
      WM_DESTROY:
        PostQuitMessage(0);
    else
      inherited;
    end;
  end;
end;

procedure TMainForm.LoadFile(const FileName: string);
var
  r: HResult;
begin
  r := Video.LoadFile(FileName);
  if r = S_OK then
  begin
    AspectMode := amAuto;
    SetTitle;
    SetFormatStr;
    if Mute then
      Video.Volume := -10000
    else
      Video.Volume := 0;
    Video.Play;
    UpdateVideoWindowSize; // ĐɃTCYXVȂƕ`悳Ȃ??
  end
  else begin
    SetTitle;
    SetFormatStr;
    OnTimer;
    FormatMsgBox(Handle, AppName, MsgCannotPlay+GetDShowErrMsg(r),
      [FileName], MB_ICONEXCLAMATION);
  end;
end;

procedure LoadFileByOtherApp(TargetWnd: HWND; const FileName: string);
var
  cds: TCopyDataStruct;
begin
  cds.dwData := 0;
  cds.cbData := Length(FileName);
  cds.lpData := PChar(FileName);
  SendMessage(TargetWnd, WM_COPYDATA, 0, LPARAM(@cds));
end;

var
  Wnd: HWND;
  i: Integer;
begin
  Wnd := AlreadyRun(AppName);
  if Wnd > -1 then
  begin
    if (Wnd > 0) and (ParamCount > 0) then
    begin
      SetForegroundWindow(Wnd);
      LoadFileByOtherApp(Wnd, ParamStr(1));
    end;
    Exit;
  end;

  CoInitialize(nil);
  try
    ProfDir := GetProfDir('NKDTPROF', 'nkvp') + '\';

    Cmd := TSCmd.Create(CmdList, DefaultKeys, ProfDir + KeyMapFile);
    MainForm := TMainForm.Create(AppName);
    with MainForm do
    begin
      CreateWnd(CW_USEDEFAULT, 0, 320, 240);
      if Handle = 0 then
        Halt(255);

      Video := TVideoPlayer.Create;
      Video.OnTimer := OnTimer;
      Video.OnEndPlay := OnEndPlay;

      Show;

      Video.DisplayWnd := Handle;

      FileList := TFileList.Create;

      for i := 1 to ParamCount do
        FileList.Add(ParamStr(i));

      if FileList.Count > 0 then
      begin
        LoadFile(FileList.CurFileName);
        Zoom := 100;
        FitWindow;
      end;
      OnTimer;

      ExitCode := MessageLoopWithAccel(Handle, Cmd.AccHandle);

      FileList.Free;
      Video.Free;
    end;
    MainForm.Free;
    Cmd.Free;
  finally
    CoUninitialize;
  end;
end.

